# -*- coding: UTF-8 -*-
##########################################################################
# TcosMonitor writen by MarioDebian <mariodebian@gmail.com>
#
#    TcosMonitor version 0.2.32-lliurex38
#
# Copyright (c) 2006 Mario Izquierdo <mariodebian@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
###########################################################################
""" template extension """

from gettext import gettext as _
import sys, os

from tcosmonitor import shared
from tcosmonitor.TcosExtensions import TcosExtension


def print_debug(txt):
    if shared.debug:
        print >> sys.stderr, "%s::%s" % (__name__, txt)
        #print("%s::%s" % (__name__, txt), file=sys.stderr)


class LockUnlockScreen(TcosExtension):
    def register(self):
        if os.path.isfile(shared.IMG_DIR + "menu_lock_custom.png"):
            img_lock="menu_lock_custom.png"
        else:
            img_lock="menu_lock.png"
        if os.path.isfile(shared.IMG_DIR + "menu_unlock_custom.png"):
            img_unlock="menu_unlock_custom.png"
        else:
            img_unlock="menu_unlock.png"
        self.main.menus.register_simple( _("Lock screen"), img_lock, 0, self.lock, "lock")
        self.main.menus.register_all( _("Lock all screens"), img_lock, 0, self.lock_all, "lock")
        self.main.menus.register_simple( _("Unlock screen"), img_unlock, 0, self.unlock, "lock")
        self.main.menus.register_all( _("Unlock all screens"), img_unlock, 0, self.unlock_all, "lock")

    def lock(self, w, ip):
        if not self.get_client():
            return
            
        self.main.worker=shared.Workers(self.main, None, None)
        self.main.worker=shared.Workers(self.main, target=self.lock_th, args=(ip,) )
        self.main.worker.start()
        
    def lock_th(self, ip):
        if not self.main.xmlrpc.lockscreen():
            shared.error_msg( _("Can't connect to tcosxmlrpc.\nPlease verify user and password in prefences!") )
            return
        self.change_lockscreen(ip)

    def unlock(self, w, ip):
        if not self.get_client():
            return
        self.main.worker=shared.Workers(self.main, None, None)
        self.main.worker=shared.Workers(self.main, target=self.unlock_th, args=(ip,) )
        self.main.worker.start()
            
    def unlock_th(self, ip): 
        if not self.main.xmlrpc.unlockscreen():
            shared.error_msg( _("Can't connect to tcosxmlrpc.\nPlease verify user and password in prefences!") )
            return
        self.change_lockscreen(ip)

    def real_action(self, ip, action):
        print_debug("real_action() ip=%s action='%s'"%(ip, action) )
        
        if action == 'lockscreen':
            self.main.xmlrpc.lockscreen()
            # update icon
            self.main.common.threads_enter("extensions/lockscreen::real_action lockscreen")
            self.change_lockscreen(ip)
            self.main.common.threads_leave("extensions/lockscreen::real_action lockscreen")
        elif action == 'unlockscreen':
            self.main.xmlrpc.unlockscreen()
            # update icon
            self.main.common.threads_enter("extensions/lockscreen::real_action unlockscreen")
            self.change_lockscreen(ip)
            self.main.common.threads_leave("extensions/lockscreen::real_action unlockscreen")


    def lock_all(self, *args):
        if not self.get_all_clients():
            return
        
        users_txt=self.connected_users_txt_all
        if (len(users_txt) > 133): users_txt="%s..." %users_txt[0:130]
        msg=_( _("Do you want to lock the following screens:\n%s?" )%(users_txt) )
        if shared.ask_msg ( msg ):
            self.main.worker=shared.Workers(self.main, None, None)
            self.main.worker.set_for_all_action(self.action_for_clients, \
                                                 self.allclients_logged, "lockscreen" )

    def unlock_all(self, *args):
        if not self.get_all_clients():
            return
            
        users_txt=self.connected_users_txt_all
        if (len(users_txt) > 133): users_txt="%s..." %users_txt[0:130]
        msg=_( _("Do you want to unlock the following screens:\n%s?" )%(users_txt) )
        if shared.ask_msg ( msg ):
            self.main.worker=shared.Workers(self.main, None, None)
            self.main.worker.set_for_all_action(self.action_for_clients, \
                                                self.allclients_logged, "unlockscreen" )


__extclass__=LockUnlockScreen






