# -*- coding: UTF-8 -*-
##########################################################################
# TcosMonitor writen by MarioDebian <mariodebian@gmail.com>
#
#    TcosMonitor version 0.2.32-lliurex38
#
# Copyright (c) 2006 Mario Izquierdo <mariodebian@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
###########################################################################
""" template extension """

from gettext import gettext as _
import sys, os

from tcosmonitor import shared
from tcosmonitor.TcosExtensions import TcosExtension


def print_debug(txt):
    if shared.debug:
        print >> sys.stderr, "%s::%s" % (__name__, txt)
        #print("%s::%s" % (__name__, txt), file=sys.stderr)


class RebootPoweroff(TcosExtension):
    def register(self):
        if os.path.isfile(shared.IMG_DIR + "menu_reboot_custom.png"):
            img_re="menu_reboot_custom.png"
        else:
            img_re="menu_reboot.png"
        if os.path.isfile(shared.IMG_DIR + "menu_poweroff_custom.png"):
            img_po="menu_poweroff_custom.png"
        else:
            img_po="menu_poweroff.png"
        self.main.menus.register_simple(_("Reboot"), img_re, 0, self.reboot, "reboot")
        self.main.menus.register_all( _("Reboot all clients"), img_re, 0, self.reboot_all, "reboot")
        self.main.menus.register_simple( _("Poweroff"), img_po, 0, self.poweroff, "reboot")
        self.main.menus.register_all( _("Poweroff all clients"), img_po, 0, self.poweroff_all, "reboot")

    def reboot(self, w, ip):
        if not self.get_client():
            return
        msg=_( _("Do you want to reboot: %s?" ) %(self.connected_users_txt_all) )
        if shared.ask_msg ( msg ):
            timeout=self.main.config.GetVar("actions_timeout")
            msg=(_("Pc will reboot in %s seconds") %timeout)
            self.main.worker=shared.Workers(self.main, None, None)
            self.main.worker=shared.Workers(self.main, target=self.exe_app_in_client, args=("reboot", timeout, msg, [ip], self.connected_users,) )
            self.main.worker.start()
    

    def reboot_all(self, *args):
        if not self.get_all_clients():
            return
            
        users_txt=self.connected_users_txt_all
        if (len(users_txt) > 133): users_txt="%s..." %users_txt[0:130]
        msg=_( _("Do you want to reboot the following clients:\n%s?" ) %(users_txt) )
        if shared.ask_msg ( msg ):
            timeout=self.main.config.GetVar("actions_timeout")
            msg=(_("Pc will reboot in %s seconds") %timeout)
            self.main.worker=shared.Workers(self.main, None, None)
            self.main.worker=shared.Workers(self.main, target=self.exe_app_in_client, args=("reboot", timeout, msg, self.allclients, self.connected_users,) )
            self.main.worker.start()

    def poweroff(self, w, ip):
        if not self.get_client():
            return
        msg=_( _("Do you want to poweroff: %s?" ) %(self.connected_users_txt_all) )
        if shared.ask_msg ( msg ):
            timeout=self.main.config.GetVar("actions_timeout")
            msg=(_("Pc will shutdown in %s seconds") %timeout)
            self.main.worker=shared.Workers(self.main, None, None)
            self.main.worker=shared.Workers(self.main, target=self.exe_app_in_client, args=("poweroff", timeout, msg, [ip], self.connected_users,) )
            self.main.worker.start()    

    def poweroff_all(self, *args):
        if not self.get_all_clients():
            return
            
        users_txt=self.connected_users_txt_all
        if (len(users_txt) > 133): users_txt="%s..." %users_txt[0:130]
        msg=_( _("Do you want to poweroff the following clients:\n%s?" )%(users_txt) )
        if shared.ask_msg ( msg ):
            timeout=self.main.config.GetVar("actions_timeout")
            msg=(_("Pc will shutdown in %s seconds") %timeout)
            self.main.worker=shared.Workers(self.main, None, None)
            self.main.worker=shared.Workers(self.main, target=self.exe_app_in_client, args=("poweroff", timeout, msg, self.allclients, self.connected_users,) )
            self.main.worker.start()

    def real_action(self, ip, action):
        print_debug("real_action() ip=%s action='%s'"%(ip, action) )
        self.main.xmlrpc.Exe(action)


    def exe_app_in_client(self, mode, timeout=0, msg="", users=[], connected_users=[]):
        remote_cmd=("/usr/lib/tcos/session-cmd-send %s %s %s" %(mode.upper(), timeout, msg.replace("'", "´")))
        action="down-controller %s %s" %(mode, timeout)
        print_debug("exe_app_in_client() usernames=%s" %users)
        
        if not users or len(users) < 1:
            return
            
        if len(connected_users) != 0 and connected_users[0] != shared.NO_LOGIN_MSG:
            newusernames=[]
            for user in connected_users:
                if user.find(":") != -1:
                    # we have a standalone user...
                    usern, ip = user.split(":")
                    self.main.xmlrpc.newhost(ip)
                    self.main.xmlrpc.DBus("exec", remote_cmd )
                else:
                    # we have a thin client
                    newusernames.append(user)
                                
            result = self.main.dbus_action.do_exec( newusernames, remote_cmd )
                
            if not result:
                shared.error_msg ( _("Error while exec remote app:\nReason:%s") %( self.main.dbus_action.get_error_msg() ) )
        
        print_debug("action_for_clients() allclients='%s' action='%s'"%(users, action))
        
        self.main.common.threads_enter("TcosActions::action_for_clients cleaning")
        self.start_action()
        self.main.progressbar.show()
        self.main.common.threads_leave("TcosActions::action_for_clients cleaning")
        
        
        for ip in users:
            if not self.doaction_onthisclient(action, ip):
                # show a msg
                print_debug( _("Can't exec this action at '%s' because you are connected at this host!") %ip )
                self.main.common.threads_enter("TcosActions::action_for_clients doing action")
                self.main.write_into_statusbar( _("Can't exec this action at '%s' because you are connected at this host!") %ip )
                self.main.common.threads_leave("TcosActions::action_for_clients doing action")
                continue
            
            
            percent=float( float(int(users.index(ip))+1)/len(users) )
            
            print_debug ( "doing %s in %s, percent complete=%f"\
                             %(action, ip, percent) )
            
            mydict={}
            mydict["action"]=action
            mydict["ip"]=ip
            self.main.common.threads_enter("TcosActions::action_for_clients doing action")
            self.main.actions.set_progressbar( _("Doing action \"%(action)s\" in %(ip)s...") %mydict , percent )
            self.main.common.threads_leave("TcosActions::action_for_clients doing action")

            if not self.main.xmlrpc.connected:
                print_debug("action_for_clients() xmlrpc not connected")
                continue
            
            try:
                # call new IP
                self.main.xmlrpc.newhost(ip)
                # overwrite real_action in your extension
                self.real_action(ip, action)
                ########################################
            except Exception, err:
                print_debug ( "action_for_clients() error while exec '%s' in %s error: %s" %(action, ip, err) )
                #pass
        
            self.main.common.threads_enter("TcosActions::action_for_clients END client")
            self.main.actions.set_progressbar( _("Done action \"%(action)s\" in %(ip)s") %mydict , 1 )
            self.main.common.threads_leave("TcosActions::action_for_clients END")
            #sleep(tcosmonitor.shared.wait_between_many_host)
        
        self.main.common.threads_enter("TcosActions::action_for_clients END all")
        self.finish_action()
        self.main.progressbar.hide()
        self.main.common.threads_leave("TcosActions::action_for_clients END all")
        return


__extclass__=RebootPoweroff






