# -*- coding: UTF-8 -*-
##########################################################################
# TcosMonitor writen by MarioDebian <mariodebian@gmail.com>
#
#    TcosMonitor version 0.2.32-lliurex38
#
# Copyright (c) 2006 Mario Izquierdo <mariodebian@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
###########################################################################
""" template extension """
from gettext import gettext as _
import sys, os

from tcosmonitor import shared
from tcosmonitor.TcosExtensions import TcosExtension

def print_debug(txt):
    if shared.debug:
        print >> sys.stderr, "%s::%s" % (__name__, txt)
        #print("%s::%s" % (__name__, txt), file=sys.stderr)


class TcosNetController(TcosExtension):
    def register(self):
        if os.path.isfile(shared.IMG_DIR + "menu_locknet_custom.png"):
            img_lock="menu_locknet_custom.png"
        else:
            img_lock="menu_locknet.png"
        if os.path.isfile(shared.IMG_DIR + "menu_unlocknet_custom.png"):
            img_unlock="menu_unlocknet_custom.png"
        else:
            img_unlock="menu_unlocknet.png"
        self.main.menus.register_simple( _("Lock internet"), img_lock, 1, self.locknet_simple, "net")
        self.main.menus.register_simple( _("Unlock internet"), img_unlock, 1, self.unlocknet_simple, "net")
        
        self.main.menus.register_all( _("Lock internet in all connected users"), img_lock, 1, self.locknet_all, "net")
        self.main.menus.register_all( _("Unlock internet in all connected users"), img_unlock, 1, self.unlocknet_all, "net")
        
    def locknet_simple(self, w, ip):
        if not self.get_client():
            return
        # lock net
        if len(self.connected_users) == 0 or self.connected_users[0] == shared.NO_LOGIN_MSG:
            shared.error_msg ( _("Can't disable internet, user is not logged") )
            return
        
        self.main.worker=shared.Workers(self.main, None, None)
        self.main.worker=shared.Workers(self.main, target=self.locknet_simple_th, args=(ip,) )
        self.main.worker.start()
        
    def locknet_simple_th(self, ip):
        print_debug("locknet_simple_th() users=%s len=%s" %(self.newallclients, len(self.newallclients)))
        eth=self.main.config.GetVar("network_interface")
        ports="--ports=%s" %self.main.config.GetVar("ports_tnc")
        remote_msg=_("Internet connection has been disabled")
        act="disable-internet"
        
        mydict={}
        mydict["action"]=_("Lock Net")
        
        self.main.common.threads_enter("TcosActions::action_for_clients cleaning")
        self.main.actions.set_progressbar( _("Doing action \"%(action)s\"...") %mydict , float(0) )
        self.main.progressbar.show()
        self.main.common.threads_leave("TcosActions::action_for_clients cleaning")
        
        if self.client_type == "tcos":
            if not self.main.localdata.user_in_group(None, 'tcos'):
                msg=(_("In order to lock and unlock internet you need to be in 'tcos' group.\n\nExe by root: adduser %s tcos" ) %(self.main.localdata.get_username()))
                shared.error_msg ( msg )
                return
            result = self.main.localdata.BlockNet(act, self.connected_users[0], ports, eth)
            if result == "disabled":
                self.main.dbus_action.do_message( self.connected_users, remote_msg )
        else:
            result = self.main.xmlrpc.tnc(act, self.connected_users[0].split(":")[0], ports)
            if result == "disabled":
                self.main.xmlrpc.DBus("mess", remote_msg)
        
        mydict["ip"]=ip
        percent=float( float(int(self.newallclients.index(ip))+1)/len(self.newallclients) )
        print_debug("locknet_simple_th() action on \"%s\" in %s percent=%s" %(mydict["action"], mydict["ip"], percent))
        self.main.common.threads_enter("TcosActions::action_for_clients doing action")
        self.main.actions.set_progressbar( _("Doing action \"%(action)s\" in %(ip)s...") %mydict , percent )
        self.main.common.threads_leave("TcosActions::action_for_clients doing action")
        
        self.change_lockscreen(ip)
        
        self.main.common.threads_enter("TcosActions::action_for_clients END all")
        self.main.progressbar.hide()
        self.main.common.threads_leave("TcosActions::action_for_clients END all")


    def unlocknet_simple(self, w, ip):
        if not self.get_client():
            return
        # unlock net
        if len(self.connected_users) == 0 or self.connected_users[0] == shared.NO_LOGIN_MSG:
            shared.error_msg ( _("Can't enable internet, user is not logged") )
            return
            
        self.main.worker=shared.Workers(self.main, None, None)
        self.main.worker=shared.Workers(self.main, target=self.unlocknet_simple_th, args=(ip,) )
        self.main.worker.start()
        
    def unlocknet_simple_th(self, ip):
        print_debug("unlocknet_simple_th() users=%s len=%s" %(self.newallclients, len(self.newallclients)))
        remote_msg=_("Internet connection has been enabled")
        act="enable-internet"
        
        mydict={}
        mydict["action"]=_("UnLock Net")
        
        self.main.common.threads_enter("TcosActions::action_for_clients cleaning")
        self.main.actions.set_progressbar( _("Doing action \"%(action)s\"...") %mydict , float(0) )
        self.main.progressbar.show()
        self.main.common.threads_leave("TcosActions::action_for_clients cleaning")
        
        if self.client_type == "tcos":
            if not self.main.localdata.user_in_group(None, 'tcos'):
                msg=(_("In order to lock and unlock internet you need to be in 'tcos' group.\n\nExe by root: adduser %s tcos" ) %(self.main.localdata.get_username()))
                shared.error_msg ( msg )
                return
            result = self.main.localdata.BlockNet(act, self.connected_users[0])
            if result == "enabled":
                self.main.dbus_action.do_message( self.connected_users, remote_msg )
        else:
            result = self.main.xmlrpc.tnc(act, self.connected_users[0].split(":")[0])
            if result == "enabled":
                self.main.xmlrpc.DBus("mess", remote_msg)
        
        mydict["ip"]=ip
        percent=float( float(int(self.newallclients.index(ip))+1)/len(self.newallclients) )
        print_debug("unlocknet_simple_th() action on \"%s\" in %s percent=%s" %(mydict["action"], mydict["ip"], percent))
        self.main.common.threads_enter("TcosActions::action_for_clients doing action")
        self.main.actions.set_progressbar( _("Doing action \"%(action)s\" in %(ip)s...") %mydict , percent )
        self.main.common.threads_leave("TcosActions::action_for_clients doing action")
        
        self.change_lockscreen(ip)
        
        self.main.common.threads_enter("TcosActions::action_for_clients END all")
        self.main.progressbar.hide()
        self.main.common.threads_leave("TcosActions::action_for_clients END all")


    def locknet_all(self, *args):
        if not self.get_all_clients():
            return
        if not self.main.localdata.user_in_group(None, 'tcos'):
            msg=(_("In order to lock and unlock internet you need to be in 'tcos' group.\n\nExe by root: adduser %s tcos" ) %(self.main.localdata.get_username()))
            shared.error_msg ( msg )
            return
        # disable internet
        
        if len(self.connected_users) == 0 or self.connected_users[0] == shared.NO_LOGIN_MSG:
            shared.error_msg( _("No users logged.") )
            return
        
        users_txt=self.connected_users_txt
        if (len(users_txt) > 133): users_txt="%s..." %users_txt[0:130]
        msg=_( _("Do you want disable internet to following users:\n%s?" )%(users_txt) )
                                                
        if not shared.ask_msg ( msg ):
            return
            
        self.main.worker=shared.Workers(self.main, None, None)
        self.main.worker=shared.Workers(self.main, target=self.locknet_all_th, args=(None,) )
        self.main.worker.start()
        
        
    def locknet_all_th(self, *args):
        print_debug("locknet_all_th() users=%s len=%s" %(self.newallclients, len(self.newallclients)))
        eth=self.main.config.GetVar("network_interface")
        ports="--ports=%s" %self.main.config.GetVar("ports_tnc")
        remote_msg=_("Internet connection has been disabled")
        act="disable-internet"
        
        mydict={}
        mydict["action"]=_("Lock Net")
        
        self.main.common.threads_enter("TcosActions::action_for_clients cleaning")
        self.main.actions.set_progressbar( _("Doing action \"%(action)s\"...") %mydict , float(0) )
        self.main.progressbar.show()
        self.main.common.threads_leave("TcosActions::action_for_clients cleaning")
                
        newusernames=[]
            
        for user in self.connected_users:
            if user.find(":") != -1:
                usern, ip=user.split(":")
                self.main.xmlrpc.newhost(ip)
                result = self.main.xmlrpc.tnc(act, usern, ports)
                if result == "disabled":
                    self.main.xmlrpc.DBus("mess", remote_msg)
            else:
                result = self.main.localdata.BlockNet(act, user, ports, eth)
                if result == "disabled":
                    newusernames.append(user)
                
            
        result = self.main.dbus_action.do_message( newusernames, remote_msg )
        
        for client in self.newallclients:
            mydict["ip"]=client
            self.main.localdata.newhost(client)
            self.main.xmlrpc.newhost(client)
            self.change_lockscreen(client)
            percent=float( float(int(self.newallclients.index(client))+1)/len(self.newallclients) )
            print_debug("locknet_all_th() action on \"%s\" in %s percent=%s" %(mydict["action"], mydict["ip"], percent))
            self.main.common.threads_enter("TcosActions::action_for_clients doing action")
            self.main.actions.set_progressbar( _("Doing action \"%(action)s\" in %(ip)s...") %mydict , percent )
            self.main.common.threads_leave("TcosActions::action_for_clients doing action")
        
        self.main.common.threads_enter("TcosActions::action_for_clients END all")
        self.main.progressbar.hide()
        self.main.common.threads_leave("TcosActions::action_for_clients END all")


    def unlocknet_all(self, *args):
        if not self.get_all_clients():
            return
        if not self.main.localdata.user_in_group(None, 'tcos'):
            msg=(_("In order to lock and unlock internet you need to be in 'tcos' group.\n\nExe by root: adduser %s tcos" ) %(self.main.localdata.get_username()))
            shared.error_msg ( msg )
            return

        if len(self.connected_users) == 0 or self.connected_users[0] == shared.NO_LOGIN_MSG:
            shared.error_msg( _("No users logged.") )
            return
        
        users_txt=self.connected_users_txt
        if (len(users_txt) > 133): users_txt="%s..." %users_txt[0:130]
        msg=_( _("Do you want enable internet to following users:\n%s?" )%(users_txt) )
                                                
        if not shared.ask_msg ( msg ):
            return
            
        self.main.worker=shared.Workers(self.main, None, None)
        self.main.worker=shared.Workers(self.main, target=self.unlocknet_all_th, args=(None,) )
        self.main.worker.start()
        
        
    def unlocknet_all_th(self, *args):
        print_debug("unlocknet_all_th() users=%s len=%s" %(self.newallclients, len(self.newallclients)))
        # enable internet
        remote_msg=_("Internet connection has been enabled")
        act="enable-internet"
        newusernames=[]
        
        mydict={}
        mydict["action"]=_("UnLock Net")
        
        self.main.common.threads_enter("TcosActions::action_for_clients cleaning")
        self.main.actions.set_progressbar( _("Doing action \"%(action)s\"...") %mydict , float(0) )
        self.main.progressbar.show()
        self.main.common.threads_leave("TcosActions::action_for_clients cleaning")
            
        for user in self.connected_users:
            if user.find(":") != -1:
                usern, ip=user.split(":")
                self.main.xmlrpc.newhost(ip)
                result = self.main.xmlrpc.tnc(act, usern)
                if result == "enabled":
                    self.main.xmlrpc.DBus("mess", remote_msg)
            else:
                result = self.main.localdata.BlockNet(act, user)
                if result == "enabled":
                    newusernames.append(user)
                
        result = self.main.dbus_action.do_message( newusernames, remote_msg )
        
        for client in self.newallclients:
            mydict["ip"]=client
            self.main.localdata.newhost(client)
            self.main.xmlrpc.newhost(client)
            self.change_lockscreen(client)
            percent=float( float(int(self.newallclients.index(client))+1)/len(self.newallclients) )
            print_debug("unlocknet_all_th() action on \"%s\" in %s percent=%s" %(mydict["action"], mydict["ip"], percent))
            self.main.common.threads_enter("TcosActions::action_for_clients doing action")
            self.main.actions.set_progressbar( _("Doing action \"%(action)s\" in %(ip)s...") %mydict , percent )
            self.main.common.threads_leave("TcosActions::action_for_clients doing action")
        
        self.main.common.threads_enter("TcosActions::action_for_clients END all")
        self.main.progressbar.hide()
        self.main.common.threads_leave("TcosActions::action_for_clients END all")


__extclass__=TcosNetController






