# -*- coding: UTF-8 -*-
##########################################################################
# TcosMonitor writen by MarioDebian <mariodebian@gmail.com>
#
#    TcosMonitor version 0.2.32-lliurex38
#
# Copyright (c) 2006 Mario Izquierdo <mariodebian@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
###########################################################################
""" template extension """
from gettext import gettext as _
import sys, os

from tcosmonitor import shared
from tcosmonitor.TcosExtensions import TcosExtension

def print_debug(txt):
    if shared.debug:
        print >> sys.stderr, "%s::%s" % (__name__, txt)
        #print("%s::%s" % (__name__, txt), file=sys.stderr)


class TcosWineController(TcosExtension):
    def register(self):
        if os.path.isfile(shared.IMG_DIR + "menu_lock_custom.png"):
            img_lock="menu_lock_custom.png"
        else:
            img_lock="menu_lock.png"
        if os.path.isfile(shared.IMG_DIR + "menu_unlock_custom.png"):
            img_unlock="menu_unlock_custom.png"
        else:
            img_unlock="menu_unlock.png"
        self.main.menus.register_all( _("Disable Wine to connected users"), img_lock, 0, self.lockwine_all, "lock")
        self.main.menus.register_all( _("Enable Wine to connected users"), img_unlock, 0, self.unlockwine_all, "lock")

    def lockwine_all(self, *args):
        if not self.get_all_clients():
            return
        if not self.main.localdata.user_in_group(None, 'tcos'):
            msg=(_("In order to lock and unlock Wine you need to be in 'tcos' group.\n\nExe by root: adduser %s tcos" ) %(self.main.localdata.get_username()))
            shared.error_msg ( msg )
            return
        
        if len(self.connected_users) == 0 or self.connected_users[0] == shared.NO_LOGIN_MSG:
            shared.error_msg( _("No users logged.") )
            return
        
        users_txt=self.connected_users_txt
        if (len(users_txt) > 133): users_txt="%s..." %users_txt[0:130]
        msg=_( _("Do you want disable Wine to following users:\n%s?" )%(users_txt) )
                                                
        if not shared.ask_msg ( msg ):
            return
            
        self.main.worker=shared.Workers(self.main, None, None)
        self.main.worker=shared.Workers(self.main, target=self.lockwine_all_th, args=(None,) )
        self.main.worker.start()
        
        
    def lockwine_all_th(self, *args):
        print_debug("lockwine_all_th() users=%s len=%s" %(self.newallclients, len(self.newallclients)))
        
        act="wine-disable"
        
        mydict={}
        mydict["action"]=_("Disable Wine")
        
        self.main.common.threads_enter("TcosActions::action_for_clients cleaning")
        self.main.actions.set_progressbar( _("Doing action \"%(action)s\"...") %mydict , float(0) )
        self.main.progressbar.show()
        self.main.common.threads_leave("TcosActions::action_for_clients cleaning")
               
        
        for client in self.newallclients:
            mydict["ip"]=client
            self.main.localdata.newhost(client)
            self.main.xmlrpc.newhost(client)
            if self.main.xmlrpc.IsStandalone(client):
                self.main.xmlrpc.wine(act, self.connected_users_txt.replace(' ','').replace('\n',''))
            percent=float( float(int(self.newallclients.index(client))+1)/len(self.newallclients) )
            print_debug("lockwine_all_th() action on \"%s\" in %s percent=%s" %(mydict["action"], mydict["ip"], percent))
            self.main.common.threads_enter("TcosActions::action_for_clients doing action")
            self.main.actions.set_progressbar( _("Doing action \"%(action)s\" in %(ip)s...") %mydict , percent )
            self.main.common.threads_leave("TcosActions::action_for_clients doing action")
        
        self.main.localdata.BlockWine(act, self.connected_users_txt.replace(' ',''))
        
        self.main.common.threads_enter("TcosActions::action_for_clients END all")
        self.main.progressbar.hide()
        self.main.common.threads_leave("TcosActions::action_for_clients END all")


    def unlockwine_all(self, *args):
        if not self.get_all_clients():
            return
        if not self.main.localdata.user_in_group(None, 'tcos'):
            msg=(_("In order to lock and unlock Wine you need to be in 'tcos' group.\n\nExe by root: adduser %s tcos" ) %(self.main.localdata.get_username()))
            shared.error_msg ( msg )
            return

        if len(self.connected_users) == 0 or self.connected_users[0] == shared.NO_LOGIN_MSG:
            shared.error_msg( _("No users logged.") )
            return
        
        msg=_( _("Do you want enable Wine to all users" ) )
                                                
        if not shared.ask_msg ( msg ):
            return
            
        self.main.worker=shared.Workers(self.main, None, None)
        self.main.worker=shared.Workers(self.main, target=self.unlockwine_all_th, args=(None,) )
        self.main.worker.start()
        
        
    def unlockwine_all_th(self, *args):
        print_debug("unlocknet_all_th() users=%s len=%s" %(self.newallclients, len(self.newallclients)))
        act="wine-enable"
        
        mydict={}
        mydict["action"]=_("Enable Wine")
        
        self.main.common.threads_enter("TcosActions::action_for_clients cleaning")
        self.main.actions.set_progressbar( _("Doing action \"%(action)s\"...") %mydict , float(0) )
        self.main.progressbar.show()
        self.main.common.threads_leave("TcosActions::action_for_clients cleaning")
        
        for client in self.newallclients:
            mydict["ip"]=client
            self.main.localdata.newhost(client)
            self.main.xmlrpc.newhost(client)
            if self.main.xmlrpc.IsStandalone(client):
                self.main.xmlrpc.wine(act, None)
            percent=float( float(int(self.newallclients.index(client))+1)/len(self.newallclients) )
            print_debug("unlockwine_all_th() action on \"%s\" in %s percent=%s" %(mydict["action"], mydict["ip"], percent))
            self.main.common.threads_enter("TcosActions::action_for_clients doing action")
            self.main.actions.set_progressbar( _("Doing action \"%(action)s\" in %(ip)s...") %mydict , percent )
            self.main.common.threads_leave("TcosActions::action_for_clients doing action")
        
        self.main.localdata.BlockWine(act, None)
        
        self.main.common.threads_enter("TcosActions::action_for_clients END all")
        self.main.progressbar.hide()
        self.main.common.threads_leave("TcosActions::action_for_clients END all")


__extclass__=TcosWineController






